import socks
import socket
import urllib2
import time
import binascii
from .exceptions import *

class Torte:
	'''
	A simple controller library for Tor that allows for all outgoing and
	incoming HTTP traffic to be funnelled through the Tor network.
	'''

	def __init__(self, tor_ip, ctl_port, **kwargs):
		'''
		Collects connection and authentication information, then
		connects to Tor.
		'''
		self.tor_ip = tor_ip
		self.ctl_port = ctl_port
		self.socks_port = kwargs.get("socks_port", ctl_port - 1)
		self.auth_pass = kwargs.get("auth_pass", None)
		self.auth_cookie = kwargs.get("auth_cookie", None)
		self.connect()

	def connect(self):
		'''
		Connect to the Tor SOCKS proxy.
		'''
		socks.setdefaultproxy(socks.PROXY_TYPE_SOCKS5, self.tor_ip, self.socks_port)
		socket.socket = socks.socksocket

	def newIP(self):
		'''
		Requests a new IP from Tor.
		'''

		socks.setdefaultproxy()
		s = socket.socket()
		s.connect((self.tor_ip, self.ctl_port))

		if self.auth_pass:
			s.send("AUTHENTICATE \"%s\"\r\n" % self.auth_pass)

		elif self.auth_cookie:
			cookie = open(self.auth_cookie, "r").read()
			s.send("AUTHENTICATE %s\r\n" % cookie.encode("hex"))

		else:
			s.send("AUTHENTICATE\r\n")

		resp = s.recv(1024)
		if resp.startswith('250'):
			s.send("SIGNAL NEWNYM\r\n")
			resp = s.recv(1024)

		s.close()
		self.connect()

	def getIP(self):
		'''
		Parses IP from HTTP connection. Time is used to avoid caching.
		'''
		url = "http://my-ip.heroku.com?t=%s" % str(time.time())
		return urllib2.urlopen(url).read().strip()

	def close(self):
		'''
		Unimplemented. Disconnect from Tor network and reset proxy
		settings.
		'''
		pass
